@@ -8,7 +8,7 @@ use RDF::RDB2RDF::R2RML;
 use RDF::RDB2RDF::DirectMapping;
 use RDF::RDB2RDF::DirectMapping::Store;
 
-our $VERSION = '0.004';
+our $VERSION = '0.005';
 
 sub new
 {
@@ -90,7 +90,7 @@ L<RDF::RDB2RDF::DirectMapping>.
 
 L<RDF::RDB2RDF::DirectMapping::Store>.
 
-L<http://perlrdf.org/>.
+L<http://www.perlrdf.org/>.
 
 =head1 AUTHOR
 
@@ -102,3 +102,10 @@ Copyright 2011 Toby Inkster
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
+
+=head1 DISCLAIMER OF WARRANTIES
+
+THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
+WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
+MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+
